// Copyright (C) 2008  Lukas Lalinsky
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#ifndef COLUMNLISTMODEL_H
#define COLUMNLISTMODEL_H

#include <QAbstractTableModel>
class Column;
class ColumnList;

class ColumnListModel : public QAbstractTableModel
{
	Q_OBJECT

public:

	enum ColumnType {
		NameColumn = 0,
		TypeColumn,
		RequiredColumn,
		PrimaryKeyColumn,
		NotesColumn,
	};

	ColumnListModel(QObject *parent = 0);

	ColumnList *columnList() const { return m_columnList; }
	void setColumnList(ColumnList *columnList);

	// QAbstractItemModel methods
	int rowCount(const QModelIndex &parent) const;
	int columnCount(const QModelIndex &parent) const;
	QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
	QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
	Qt::ItemFlags flags(const QModelIndex &index) const;
	bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole);

	QModelIndex indexFromRow(int i) const;

protected:
	virtual int firstRow() const { return 0; }

private slots:
	void _columnAboutToBeInserted(int index);
	void _columnInserted();
	void _columnAboutToBeRemoved(int index);
	void _columnRemoved();
	void _columnChanged(int index);

private:
	ColumnList *m_columnList;
};

#endif
